@@ -6,9 +6,13 @@ package WebDAO::Response;
 
 WebDAO::Response - Response class
 
+=head1 SYNOPSYS
+
+        new WebDAO::Response:: cv => $cv
+
 =head1 DESCRIPTION
 
-Class for set response headers
+Class for make HTTP response
 
 =cut
 
@@ -18,8 +22,21 @@ use IO::File;
 use DateTime;
 use DateTime::Format::HTTP;
 use base qw( WebDAO::Base );
-__PACKAGE__->attributes
-  qw/  __session _headers _is_headers_printed _cv_obj _is_file_send _is_need_close_fh __fh _is_flushed _call_backs/;
+
+__PACKAGE__->mk_attr( 
+    _headers => undef,
+    _is_headers_printed =>0,
+    _cv_obj => undef,
+    _is_file_send => 0,
+    _is_need_close_fh => 0,
+    __fh => undef,
+    _is_flushed => 0,
+    _call_backs => undef,
+    _is_modal => 0,
+    _forced_want_format => undef,
+    _is_empty=>0,
+    status => 200 #default HTTP status
+    );
 
 use strict;
 
@@ -27,7 +44,7 @@ use strict;
 
 =cut
 
-sub _init() {
+sub _init {
     my $self = shift;
     return $self->init(@_);
 }
@@ -38,16 +55,43 @@ sub init {
     $self->_headers( {} );
     $self->_call_backs( [] );
     $self->_cv_obj( $par{cv} );
-    $self->__session( $par{session} );
     return 1;
 }
 
+=head2 get_request
+
+Return ref to request object (WebDAO::CV)
+
+=cut
+
+sub get_request {
+    my $self = shift;
+    return $self->_cv_obj;
+}
+
+=head2 set_status INT
+
+set response HTTP status
+
+    $r->set_status(200)
+
+return C<$self>
+
+=cut
+
+sub set_status {
+    my $self = shift;
+    $self->status(@_);
+    $self 
+}
+
+
 =head2 set_header NAME, VALUE
 
 Set out header:
 
         $response->set_header('Location', $redirect_url);
-        $response->set_header( -type => 'text/html; charset=utf-8' );
+        $response->set_header( 'Content-Type' => 'text/html; charset=utf-8' );
 
 return $self reference
 
@@ -55,12 +99,31 @@ return $self reference
 
 sub set_header {
     my ( $self, $name, $par ) = @_;
-
-    #    $self->_headers->{ $name =~ /^-/ ? uc $name : $name } = $par;
-    $self->_headers->{ uc $name } = $par;
+    #translate CGI headers
+    if ( $name =~ /^-/) {
+            my $UKey = uc $name;
+            
+            if ( $UKey eq '-STATUS' ) {
+                my ($status) = $par =~ m/(\d+)/;
+                $self->status($status);
+                return; #don't save status
+            }
+            warn "Deprecated header name $name !";
+#            use CGI;
+#            my $h = CGI->new->header( $UKey, $par );
+#            $h =~ s/\015\012//g;
+#            ( $name, $par ) = split( /\s*:\s*/, $h );
+    } elsif ( $name eq 'Set-Cookie') {
+        push @{ $self->_headers->{ $name } }, $par;
+        return $self
+    }
+        
+    $self->_headers->{ $name } = $par;
     $self;
 }
 
+
+
 =head2 get_header NAME
 
 return value for  header NAME:
@@ -69,9 +132,41 @@ return value for  header NAME:
 
 sub get_header {
     my ( $self, $name ) = @_;
-    return $self->_headers->{ uc $name };
+    return $self->_headers->{ $name };
 }
 
+=head2 aliases for headers
+
+=head3 content_type
+
+  $r->content_type('text/html; charset=utf-8');
+
+=cut
+
+sub content_type {
+    my $self = shift;
+    unless ($#_ > 0 ) {
+        $self->set_header('Content-Type', @_)
+    }
+    $self->get_header('Content-Type');
+}
+
+=head3 content_length
+
+A decimal number indicating the size in bytes of the message content.
+
+=cut
+
+sub content_length {
+    my $self = shift;
+    unless ($#_ > 0 ) {
+      $self->set_header('Content-Length' , @_)
+    } 
+    $self->get_header('Content-Length');
+}
+
+=head3  
+
 =head2 get_mime_for_filename <filename>
 
 Determine mime type for filename (Simple by ext);
@@ -82,6 +177,7 @@ return str
 sub get_mime_for_filename {
     my $self          = shift;
     my $filename      = shift;
+    my $no_default_flag = shift;
     my %types_for_ext = (
         avi  => 'video/x-msvideo',
         bmp  => 'image/bmp',
@@ -114,7 +210,7 @@ sub get_mime_for_filename {
     if ( my $type = $types_for_ext{ lc $ext } ) {
         return $type;
     }
-    return 'application/octet-stream';
+    return $no_default_flag ? undef : 'application/octet-stream';
 }
 
 =head2 print_header
@@ -127,47 +223,39 @@ sub print_header {
     my $self  = shift;
     my $pnted = $self->_is_headers_printed;
     return $self if $pnted;
-    my $res     = { data => '' };    #need for cv->response
-    my $cv      = $self->_cv_obj;
-    my $headers = $self->_headers;
-    $headers->{-TYPE} = $res->{type} if $res->{type};    #deprecated
-    while ( my ( $key, $val ) = each %$headers ) {
-        my $UKey = uc $key;
-        $res->{headers}->{$UKey} = $headers->{$UKey}
-          unless exists $res->{headers}->{$UKey};
-    }
-    $cv->response($res);
+    my $cv      = $self->get_request;
+    $cv->status($self->status);
+    $cv->print_headers(%{ $self->_headers });
     $self->_is_headers_printed(1);
     $self;
 }
 
-=head2  redirect2url <url for redirect to>
+=head2  redirect2url <url for redirect to> [, $code]
 
 Set headers for redirect to url.return $self reference
 
 =cut
 
 sub redirect2url {
-    my ( $self, $redirect_url ) = @_;
-    $self->set_header( "-status",   '302 Found' );
-    $self->set_header( '-Location', $redirect_url );
+    my ( $self, $redirect_url, $code ) = @_;
+    $self->set_modal->set_status( $code || 302 );
+    $self->set_header( 'Location', $redirect_url );
 }
 
-=head2 set_cookie ( -name => <name>, ...)
+=head2 set_cookie ( name => <cookie_name>, value=><cookie_value> ...)
 
-Set cookie. For params see manpage for  CGI::cookie.
+Set cookie.
 return $self reference
 
 =cut
 
 sub set_cookie {
     my $self = shift;
-    my $res  = $self->get_header( -cookie ) || [];
-    my $cv   = $self->_cv_obj;
-    push @$res, $cv->cookie(@_);
-    return $self->set_header( -cookie => $res );
+    $self->set_header("Set-Cookie", { @_ });
+    $self;
 }
 
+
 =head2 set_callback(sub1{}[, sub2{} ..])
 
 Set callbacks for call after flush
@@ -210,30 +298,32 @@ sub send_file {
 
     #set file headers
     my ( $size, $mtime ) = ( stat $file_handle )[ 7, 9 ];
-    $self->set_header( '-Content_length', $size );
+    $self->content_length( $size );
     my $formated =
       DateTime::Format::HTTP->format_datetime(
         DateTime->from_epoch( epoch => $mtime ) );
-    $self->set_header( '-Last-Modified', $formated );
+    $self->set_header( 'Last-Modified', $formated );
 
     #Determine mime tape of file
     if ( my $predefined = $args{-type} ) {
-        $self->set_header( -type => $predefined );
+        $self->content_type( $predefined );
     }
     else {
         ##
         if ($file_name) {
-            $self->set_header(
-                -type => $self->get_mime_for_filename($file_name) );
+            $self->content_type(
+                $self->get_mime_for_filename($file_name) );
         }
     }
-    $self->_is_file_send(1);
+
+    #set modal mode and flag for send file
+    $self->set_modal->_is_file_send(1);
     $self;
 }
 
 sub print {
     my $self = shift;
-    my $cv   = $self->_cv_obj;
+    my $cv   = $self->get_request;
     $self->print_header;
     $cv->print(@_);
     return $self;
@@ -259,7 +349,7 @@ sub flush {
     #do self print file
     if ( $self->_is_file_send ) {
         my $fd = $self->__fh;
-        $self->_cv_obj->print(<$fd>);
+        $self->get_request->print(<$fd>);
         close($fd) if $self->_is_need_close_fh;
     }
     $self->_is_flushed(1);
@@ -275,6 +365,18 @@ sub flush {
     $self;
 }
 
+=head2 set_modal
+
+Set modal mode for answer
+
+=cut
+
+sub set_modal {
+    my $self = shift;
+    $self->_is_modal(1);
+    $self;
+}
+
 =head2 error404
 
 Set HTTP 404 headers
@@ -283,7 +385,7 @@ Set HTTP 404 headers
 
 sub error404 {
     my $self = shift;
-    $self->set_header( "-status", '404 Not Found' );
+    $self->set_modal->set_status(404);
     $self->print(@_) if @_;
     return $self;
 }
@@ -305,6 +407,7 @@ sub json : lvalue {
     $self->{__json};
 
 }
+
 sub set_json {
     my $self = shift;
     my $data = shift;
@@ -315,14 +418,83 @@ sub set_json {
 sub _destroy {
     my $self = shift;
     $self->{__html} = undef;
-
-    #    $self->_headers( {} );
-    #    $self->_call_backs( [] );
-    #    $self->_cv_obj( undef );
-    $self->__session(undef);
-
-    #    $self->auto( [] );
+    #destroy called from Engine::execute2
+    # destroy tests by cleared _cv_obj
+#    $self->_cv_obj( undef );
+    $self->_headers( {} );
+    $self->_call_backs( [] );
 }
+
+=head2 wantformat ['format',['forse_set_format']]
+
+Return expected output format: defauilt html
+    
+       # return string for format
+       $r->wantformat()
+
+Check if desired format is expected
+
+  #$r->wantformat('html') return boolean
+  if ($r->wantformat('html')) { 
+      # 
+  }
+
+Force set desired format:
+
+  $r->wantformat('html'=>1); #return $response object ref
+
+=cut
+
+sub wantformat {
+    my $self = shift;
+    if ( @_ > 1 ) {
+        $self->_forced_want_format(shift);
+        return $self;
+    }
+    my $desired = $self->_forced_want_format();
+    my $default =
+         $desired
+      || $self->detect_wantformat( $self->get_request ) #call with CV object 
+      || 'html';
+    if ( scalar(@_) == 1 ) {
+        return $default eq shift;
+    }
+    return $default;
+}
+
+=head2 detect_wantformat ($cv)
+
+Method for detect output format when C<wantformat()> called
+
+Must return :
+    
+        string  - output format, i.e. 'html', 'xml'
+        undef - unknown ( use defaults )
+
+=cut
+
+sub detect_wantformat {
+    return undef    #unknown by default
+}
+
+
+=head2 set_empty
+
+Set flag for empty response. Headers are not printed.
+return $self
+=cut
+
+sub set_empty { $_[0]->_is_empty(1); $_[0]}
+
+=head2 is_empty
+
+Check is response cleared.
+Return 1|0
+
+=cut
+
+sub is_empty { return $_[0]->_is_empty() }
+
 1;
 __DATA__
 
@@ -336,7 +508,7 @@ Zahatski Aliaksandr, E<lt>zag@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2002-2010 by Zahatski Aliaksandr
+Copyright 2002-2012 by Zahatski Aliaksandr
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
