@@ -15,6 +15,7 @@ use WebDAO;
 use WebDAO::SessionSH;
 use WebDAO::Store::Abstract;
 use Data::Dumper;
+use WebDAO::Lex;
 use Getopt::Long;
 use Pod::Usage;
 
@@ -29,20 +30,11 @@ sub _parse_str_to_hash {
     \%hash;
 }
 
-my ( $store_class, $session_class, $eng_class ) = map {
-    eval "require $_"
-      or die $@;
-    $_
-  } (
-    $ENV{wdStore}   || 'WebDAO::Store::Abstract',
-    $ENV{wdSession} || 'WebDAO::SessionSH',
-    $ENV{wdEngine}  || 'WebDAO::Engine'
-  );
 
 my ( $help, $man, $sess_id );
 my %opt = ( help => \$help, man => \$man, sid => \$sess_id );   #meta=>\$meta,);
 my @urls=();
-GetOptions( \%opt, 'help|?', 'man', 'f=s', 'sid|s=s','<>'=>sub { push @urls,shift} )
+GetOptions( \%opt, 'help|?', 'man', 'f=s','wdEngine|M=s','wdEnginePar=s', 'sid|s=s','<>'=>sub { push @urls,shift} )
   or pod2usage(2);
 pod2usage(1) if $help;
 pod2usage( -exitstatus => 0, -verbose => 2 ) if $man;
@@ -67,6 +59,16 @@ foreach my $sname ('__DIE__') {
       }
 }
 
+my ( $store_class, $session_class, $eng_class ) = map {
+    eval "require $_"
+      or die $@;
+    $_
+  } (
+    $ENV{wdStore}   || 'WebDAO::Store::Abstract',
+    $ENV{wdSession} || 'WebDAO::SessionSH',
+    $opt{wdEngine}|| $ENV{wdEngine}  || 'WebDAO::Engine',
+  );
+
 #Make Session object
 my $store_obj =
   $store_class->new( %{ &_parse_str_to_hash( $ENV{wdStorePar} ) || {} } );
@@ -74,26 +76,29 @@ my $sess = $session_class->new(
         %{ &_parse_str_to_hash( $ENV{wdSessionPar} ) || {} },
         store => $store_obj,
 );
-#$sess->U_id($sess_id);
+$sess->U_id($sess_id);
 my ($filename) = grep { -r $_ && -f $_ } $ENV{wdIndexFile} || $opt{f};
 die "$0 ERR:: file not found or can't access (wdIndexFile): $ENV{wdIndexFile}"
   unless $filename;
 
-
-my $content = qq!<wD><include file="$filename"/></wD>!;
-my $lex = new WebDAO::Lex:: content => $content;
+open FH, "<$filename" or die $!;
+my $content ='';
+{ local $/=undef;
+$content = <FH>;
+}
+close FH;
+my $lex = new WebDAO::Lex:: tmpl => $content;
 my $eng = $eng_class->new(
-    %{ &_parse_str_to_hash( $ENV{wdEnginePar} ) || {} },
-    lexer    => $lex,
+    %{ &_parse_str_to_hash( $opt{wdEnginePar} || $ENV{wdEnginePar} ) || {} },
+    lex    => $lex,
     session  => $sess,
 );
-$sess->ExecEngine($eng);
+
+$sess->ExecEngine($eng, $evl_file);
 $sess->destroy;
 croak STDERR $@ if $@;
 print "\n";
 
-#$sess->ExecEngine($eng);
-
 =head1 NAME
 
   wd_shell.pl  - command line tool for developing and debuging
@@ -140,7 +145,7 @@ Zahatski Aliaksandr, E<lt>zag@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2000-2006 by Zahatski Aliaksandr
+Copyright 2000-2011 by Zahatski Aliaksandr
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
